@@ -1141,6 +1141,7 @@ class MetadynamicSimulation(MDSimulation):
             print('Adding a flat-bottom restraint with radius: {} '.format(self.dome_radius))
 
             fb_force = mm.CustomCentroidBondForce(2, 'step(r-r0) * (fb_k) * (r-r0)^2; r=distance(g1,g2)')
+            fb_force.setUsesPeriodicBoundaryConditions(True)
             fb_force.addGlobalParameter('r0',self.dome_radius)
             fb_force.addGlobalParameter('fb_k',20)
             # find anchor group
@@ -1198,6 +1199,7 @@ class MetadynamicSimulation(MDSimulation):
                 'K*max(0, (r-radius))^2; r=sqrt(periodicdistance(x,y,z*0,x0,y0,zcc*0)^2); '
                 'radius=min(r_cyl,(tan(alpha)*orient_factor*periodicdistance(x*0,y*0,z,x0*0,y0*0,zcc)))')
 
+            funnel_force.setUsesPeriodicBoundaryConditions(True)
             funnel_force.addGlobalParameter('K', 10)
             funnel_force.addGlobalParameter('r_cyl', r_cyl)
             funnel_force.addGlobalParameter('zcc', z_cone_apex)
