@@ -142,8 +142,8 @@ class SegmentMappingSim(MetadynamicSimulation):
                     for match in matches:
                         os.remove(os.path.join(self.cwd, match.string))
 
+                self.production_CV_list = self.CV_list
                 if self.scan_CV_list is not None:
-                    self.production_CV_list = self.CV_list
                     self.CV_list = self.scan_CV_list[i]
                     self._remove_custom_forces()
                     self._build_custom_forces()
@@ -451,8 +451,7 @@ class SegmentMappingSimHREMD(SegmentMappingSim):
                 print('current cv1 distance', current_cv_values)
             #compare deltaU of swap and normalize high energy probability of acceptance by 2.5 (energy of room temp in kJ/mol)
             if log_p_accept<=0 * unit.kilojoules_per_mole \
-                    or random.random() < np.exp(-(log_p_accept/2.494341741366065).value_in_unit(unit.kilojoule_per_mole)) \
-                    or current_cv_values[0]>self.unbound_distance:
+                    or random.random() < np.exp(-(log_p_accept/2.494341741366065).value_in_unit(unit.kilojoule_per_mole)):
                 accepted +=1
                 attempted +=1
             else:
