@@ -1,3 +1,10 @@
+from simtk.openmm import unit, app, LangevinIntegrator, Platform
+import mdtraj
+
+from copy import deepcopy
+import parmed
+
+
 
 import numpy as np
 from scipy import signal, spatial
@@ -589,4 +596,200 @@ def get_minimum_path_trajectory(fes,CV_trace,CV_list,traj,order,energy_cutoff,fr
         final_list = np.unique(np.concatenate(final_list))
         print(final_list)
         chosen_traj = traj[final_list]
-        return chosen_traj, frame_list
\ No newline at end of file
+        return chosen_traj, frame_list
+
+
+
+def calc_mmgb(parmed, traj, sel, implicit_model):
+    """
+
+    Args:
+        parmed:
+        traj:
+        sel:
+        implicit_model:
+
+    Returns:
+        np.array
+    """
+
+    # build openmm system to calculate the energy
+    simulation = build_omm_system(parmed, implicit_model=implicit_model)
+
+    # loop across the frames in the traj to calculate the mmgb energy
+    num_frames = traj.xyz.shape[0]
+    coords = traj.xyz[:,sel,:]
+
+    mm_list = []
+    for i in range(num_frames):
+        simulation.context.setPositions(coords[i] * unit.nanometer)
+        state = simulation.context.getState(getEnergy=True)
+        frame_energy = state.getPotentialEnergy().value_in_unit(unit.kilocalorie_per_mole)
+        mm_list.append(float(frame_energy))
+        # print('frame {} energy {}'.format(i,frame_energy))
+
+    mm_array = np.array(mm_list)
+    return mm_array
+
+
+def build_omm_system(parmed, implicit_model=None):
+    """
+
+    Args:
+        parmed:
+        implicit_model:
+
+    Returns:
+        simtk.openmm.app.simulation
+    """
+    system = parmed.createSystem(nonbondedMethod=app.NoCutoff)
+
+    if implicit_model is not None:
+        implicit_model_object = eval("app.%s" % implicit_model)
+        implicit_force = parmed.omm_gbsa_force(implicit_model_object,
+                                               temperature=300 * unit.kelvin,
+                                               nonbondedMethod=app.NoCutoff,
+                                               nonbondedCutoff=1 * unit.nanometer)
+        system.addForce(implicit_force)
+
+    integrator = LangevinIntegrator(300.0 * unit.kelvin, 1 / unit.picoseconds, 0.002 * unit.picoseconds)
+    simulation = app.Simulation(parmed.topology, system, integrator, platform=Platform.getPlatformByName('CPU'))
+
+    return simulation
+
+
+def calc_mm(parmed, traj, sel):
+    """
+
+    Args:
+        parmed:
+        traj:
+        sel:
+
+    Returns:
+        np.array
+    """
+
+    # build openmm system to calculate the energy
+    simulation = build_omm_system(parmed)
+
+    # loop across the frames in the traj to calculate the mmgb energy
+    num_frames = traj.xyz.shape[0]
+    coords = traj.xyz[:, sel, :]
+
+    mm_list = []
+    for i in range(num_frames):
+        simulation.context.setPositions(coords[i] * unit.nanometer)
+        state = simulation.context.getState(getEnergy=True)
+        frame_energy = state.getPotentialEnergy().value_in_unit(unit.kilocalorie_per_mole)
+        mm_list.append(float(frame_energy))
+        # print('frame {} energy {}'.format(i,frame_energy))
+
+    mm_array = np.array(mm_list)
+    return mm_array
+
+
+# Note! using calc_gb and calc_mm is more expensive than using calc_mmgb
+def calc_gb(parmed, traj, sel, implicit_model):
+    """
+
+    Args:
+        parmed: Parmed.structure
+        traj: MDTraj.trajectory
+        sel: list of integers
+        implicit_model: {'HCT', 'OBC1', 'OBC2'}
+
+    Returns:
+        np.array
+    """
+
+    simulation = build_omm_system(parmed, implicit_model=implicit_model)
+    mm_array = calc_mm(parmed, traj, sel)
+
+    # loop across the frames in the traj to calculate the mmgb energy
+    num_frames = traj.xyz.shape[0]
+    coords = traj.xyz[:, sel, :]
+
+    mmgb_list = []
+    for i in range(num_frames):
+        simulation.context.setPositions(coords[i] * unit.nanometer)
+        state = simulation.context.getState(getEnergy=True)
+        frame_energy = state.getPotentialEnergy().value_in_unit(unit.kilocalorie_per_mole)
+        mmgb_list.append(float(frame_energy))
+
+    gb_array = np.array(mmgb_list) - mm_array
+
+    return gb_array
+
+
+def calc_sa(traj, sel):
+    """
+
+    Args:
+        traj:
+        sel:
+
+    Returns:
+        np.array
+    """
+    # now get sasa from mdtraj
+
+    temp_traj = deepcopy(traj)
+    temp_traj.restrict_atoms(sel)
+
+    sasa = mdtraj.shrake_rupley(temp_traj)
+    sasa = sasa.sum(axis=1)
+
+    sasa_energy = sasa * -.6  # surface tension of water in kcal/mol/nm^2
+
+    return sasa_energy
+
+def calc_mmgbsa(parmed_structure,traj):
+
+    complex_sel = select_atoms(parmed_structure=parmed_structure,keyword_selection='protein ligand')
+    protein_sel = select_atoms(parmed_structure=parmed_structure,keyword_selection='protein')
+    ligand_sel = select_atoms(parmed_structure=parmed_structure,keyword_selection='ligand')
+
+    complex_parmed = parmed_structure[complex_sel]
+    protein_parmed = parmed_structure[protein_sel]
+    ligand_parmed = parmed_structure[ligand_sel]
+
+    complex_mmgb = calc_mmgb(complex_parmed, traj, complex_sel, 'HCT')
+    prot_mmgb = calc_mmgb(protein_parmed, traj, protein_sel, 'HCT')
+    lig_mmgb = calc_mmgb(ligand_parmed, traj, ligand_sel, 'HCT')
+
+    complex_sa = calc_sa(traj, complex_sel)
+    prot_sa = calc_sa(traj, protein_sel)
+    lig_sa = calc_sa(traj,ligand_sel)
+
+    final_mmgb = complex_mmgb - (prot_mmgb + lig_mmgb)
+    final_sa = complex_sa - (prot_sa + lig_sa)
+
+    mmgbsa = final_mmgb + final_sa
+    mmgbsa_array = np.array(mmgbsa)
+
+    mmgbsa = np.mean(mmgbsa_array)
+    mmgbsa_std = np.std(mmgbsa_array)
+    return mmgbsa, mmgbsa_std
+
+
+def calc_rmsf(traj: mdtraj.Trajectory,lig_resname: str):
+    """
+    accepts a trajectory of a bound ligand and calculates the average ligand displacement wrt to the protein
+    :param traj:
+    :param lig_resname:
+    :return rmsf (float):
+    """
+
+    traj.center_coordinates()
+
+    # superpose alignment selection to first frame
+    traj.superpose(traj[0], atom_indices=traj.top.select('protein'))
+
+    atomsel = traj.top.select('resname '+lig_resname)
+    atomsel = np.array(atomsel).astype(int)
+
+    average_xyz = np.array([np.average(traj.xyz[:, atomsel, :], axis=0)])
+    rmsf = np.average(
+        np.sqrt(np.sum(np.square(traj.xyz[:, atomsel, :] - average_xyz), axis=(0, 2)) / traj.xyz.shape[0]))
+    return rmsf
\ No newline at end of file
